@@ -1,6 +1,8 @@
 context("rma.mv objects")
 set.seed(20190513)
 
+skip_if_not_installed("robumeta")
+skip_if_not_installed("metafor")
 
 CR_types <- paste0("CR",0:4)
 
@@ -156,7 +158,7 @@ test_that("clubSandwich works with missing diagonal variances", {
 
 test_that("clubSandwich works with missing vcov matrix", {
   
-  skip_if(packageVersion("metafor") < 2.1)
+  skip_if(packageVersion("metafor") < "2.1")
   
   dat_miss <- corrdat
   dat_miss$var[sample.int(nrow(corrdat), size = round(nrow(corrdat) / 10))] <- NA
@@ -165,12 +167,14 @@ test_that("clubSandwich works with missing vcov matrix", {
   
   expect_warning(corr_drop <- rma.mv(effectsize ~ males + college + binge, 
                                      random = ~ 1 | studyid, 
-                                     V = V_missing, data = dat_miss))
+                                     V = V_missing, data = dat_miss,
+                                     sparse = TRUE))
   
   corr_complete <- rma.mv(effectsize ~ males + college + binge,
                           random = ~ 1 | studyid, 
                           subset = !is.na(var),
-                          data = dat_miss, V = V_missing)
+                          data = dat_miss, V = V_missing,
+                          sparse = TRUE)
   
   expect_error(vcovCR(corr_complete, type = "CR0", cluster = dat_miss$studyid))
   
@@ -178,26 +182,6 @@ test_that("clubSandwich works with missing vcov matrix", {
   CR_complete <- lapply(CR_types, function(x) vcovCR(corr_complete, type = x))
   expect_equal(CR_drop, CR_complete, tolerance = 1e-5)
   
-  
-  # V_complete <- impute_covariance_matrix(corrdat$var, cluster = corrdat$studyid, r = 0.8)
-  # W_missing <- lapply(V_complete, function(x) chol2inv(chol(x)))
-  # 
-  # corr_drop <- rma.mv(effectsize ~ males + college + binge, 
-  #                     random = ~ 1 | studyid, 
-  #                     V = V_complete, W = bldiag(W_missing), 
-  #                     data = dat_miss)
-  # 
-  # corr_complete <- rma.mv(effectsize ~ males + college + binge,
-  #                         random = ~ 1 | studyid, 
-  #                         V = V_complete, W = bldiag(W_missing),
-  #                         data = dat_miss, subset = !is.na(var))
-  # 
-  # expect_error(vcovCR(corr_complete, type = "CR0", cluster = dat_miss$studyid))
-  # 
-  # CR_drop <- lapply(CR_types, function(x) vcovCR(corr_drop, type = x))
-  # CR_complete <- lapply(CR_types, function(x) vcovCR(corr_complete, type = x))
-  # expect_equal(CR_drop, CR_complete)
-  
 })
 
 
@@ -233,7 +217,8 @@ test_that("clubSandwich works with complicated random effects specifications.",
   m1 <- rma.mv(
     R ~ 0 + IAT.Focus + Crit.Cat, V = V,
     data = oswald2013,
-    random = list(~ 1 | Study, ~ 1 | SSID, ~ 1 | ESID)
+    random = list(~ 1 | Study, ~ 1 | SSID, ~ 1 | ESID),
+    sparse = TRUE
   )
   
   m2 <- update(m1, 
@@ -265,7 +250,8 @@ test_that("clubSandwich works with complicated random effects specifications.",
     R ~ 0 + IAT.Focus + Crit.ID, V = V,
     data = oswald2013,
     random = list(~ 1  + Crit.ID | Study),
-    struct = c("GEN")
+    struct = c("GEN"),
+    sparse = TRUE
   )
   
   m12 <- update(m11, random = list(~ 1  + Crit.ID | Study, ~ 1 | SSID))
@@ -305,7 +291,8 @@ test_that("clubSandwich works for random slopes model.", {
   dat$vi <- dat$vi*100^2
   res <- rma.mv(yi, vi, mods = ~ bmicent, 
                 random = ~ bmicent | study, struct="GEN", 
-                data=dat)
+                data=dat,
+                sparse = TRUE)
   
   cl <- findCluster.rma.mv(res)
   
@@ -324,17 +311,23 @@ test_that("clubSandwich works for correlated hierarchical effects model.", {
                                     smooth_vi = TRUE)
   
   CHE_es <- rma.mv(effectsize ~ males + college + binge, data = corrdat, 
-                   V = V_mat, random = ~ 1 | esid)
+                   V = V_mat, random = ~ 1 | esid,
+                   sparse = TRUE)
   CHE_study <- rma.mv(effectsize ~ males + college + binge, data = corrdat, 
-                      V = V_mat, random = ~ 1 | studyid)
+                      V = V_mat, random = ~ 1 | studyid,
+                      sparse = TRUE)
   CHE_studyes <- rma.mv(effectsize ~ males + college + binge, data = corrdat, 
-                        V = V_mat, random = ~ 1 | studyid / esid)
+                        V = V_mat, random = ~ 1 | studyid / esid,
+                        sparse = TRUE)
   CHE_esstudy <- rma.mv(effectsize ~ males + college + binge, data = corrdat, 
-                        V = V_mat, random = ~ 1 | esid/ studyid)
+                        V = V_mat, random = ~ 1 | esid/ studyid,
+                        sparse = TRUE)
   CHE_study_es <- rma.mv(effectsize ~ males + college + binge, data = corrdat, 
-                         V = V_mat, random = list(~ 1 | studyid,  ~ 1 | esid))
+                         V = V_mat, random = list(~ 1 | studyid,  ~ 1 | esid),
+                         sparse = TRUE)
   CHE_es_study <- rma.mv(effectsize ~ males + college + binge, data = corrdat, 
-                         V = V_mat, random = list(~ 1 | esid, ~ 1 | studyid))
+                         V = V_mat, random = list(~ 1 | esid, ~ 1 | studyid),
+                         sparse = TRUE)
   
   mods <- list(es = CHE_es, study = CHE_study, 
                studyes = CHE_studyes, esstudy = CHE_esstudy,
@@ -407,7 +400,8 @@ test_that("clubSandwich works when random effects variable has missing levels.",
   
   mlma_fac <- rma.mv(yi ~ year, V = vi, 
                      random = ~ 1 | district_fac / study,
-                     sparse = TRUE, data = dat)
+                     data = dat,
+                     sparse = TRUE)
   
   implicit_fac <- coef_test(mlma_fac, vcov = "CR2")
   explicit_fac <- coef_test(mlma_fac, vcov = "CR2", cluster = dat$district_fac)
@@ -415,7 +409,8 @@ test_that("clubSandwich works when random effects variable has missing levels.",
 
   mlma_plus <- rma.mv(yi ~ year, V = vi, 
                      random = ~ 1 | district_fac_plus / study,
-                     sparse = TRUE, data = dat)
+                     data = dat,
+                     sparse = TRUE)
   
   implicit_plus <- coef_test(mlma_plus, vcov = "CR2")
   explicit_plus <- coef_test(mlma_plus, vcov = "CR2", cluster = dat$district_fac_plus)
@@ -425,7 +420,8 @@ test_that("clubSandwich works when random effects variable has missing levels.",
   
   mlma_num <- rma.mv(yi ~ year, V = vi, 
                      random = ~ 1 | district / study,
-                     sparse = TRUE, data = dat)
+                     data = dat,
+                     sparse = TRUE)
   
   implicit_num <- coef_test(mlma_num, vcov = "CR2")
   explicit_num <- coef_test(mlma_num, vcov = "CR2", cluster = dat$district)
@@ -531,68 +527,24 @@ test_that("clubSandwich methods work on robust.rma objects.", {
   
 })
 
-test_that("clubSandwich works with user-weighted rma.mv objects.", {
+test_that("clubSandwich works with test = 'knha'.", {
   
   data("oswald2013", package = "robumeta")
   oswald2013$yi <- atanh(oswald2013$R)
   oswald2013$vi <- 1 / (oswald2013$N - 3)
-  oswald2013$esID <- 1:nrow(oswald2013)
-  oswald2013$wt <- 1 + rpois(nrow(oswald2013), lambda = 1)
   
-  V <- impute_covariance_matrix(vi = oswald2013$vi, cluster = oswald2013$Study, r = 0.4)
-  
-  mod_wt1 <- rma.mv(yi ~ 0 + Crit.Cat + Crit.Domain + IAT.Focus + Scoring,
-                    V = V, W = wt,
-                    random = ~ 1 | Study,
+  mod1 <- rma.mv(yi ~ 0 + Crit.Cat + Crit.Domain + IAT.Focus + Scoring,
+                    V = vi, 
                     data = oswald2013,
                     sparse = TRUE)
   
-  W_mat <- impute_covariance_matrix(vi = oswald2013$wt, cluster = oswald2013$Study, r = 0, return_list = FALSE)
+  suppressWarnings(
+    mod2 <- update(mod1, test = "knha")
+  )
   
-  mod_wt2 <- rma.mv(yi ~ 0 + Crit.Cat + Crit.Domain + IAT.Focus + Scoring,
-                    V = V, W = W_mat,
-                    random = ~ 1 | Study,
-                    data = oswald2013,
-                    sparse = TRUE)
-  
-  
-  vcovs_1 <- lapply(CR_types, function(x) vcovCR(mod_wt1, type = x))
-  vcovs_2 <- lapply(CR_types, function(x) vcovCR(mod_wt2, type = x))
+  vcovs_1 <- lapply(CR_types, function(x) vcovCR(mod1, type = x, cluster = oswald2013$Study))
+  vcovs_2 <- lapply(CR_types, function(x) vcovCR(mod2, type = x, cluster = oswald2013$Study))
 
-  coef_test_wt1 <- lapply(CR_types, function(x) 
-    coef_test(mod_wt1, vcov = x, test = "All")
-  )
-  
-  coef_test_wt2 <- lapply(CR_types, function(x) 
-    coef_test(mod_wt2, vcov = x, test = "All")
-  )
-  
-  Wald_test_wt1 <- lapply(CR_types, function(x) 
-    Wald_test(mod_wt1, 
-              constraints = constrain_equal("Crit.Cat", reg_ex = TRUE),
-              vcov = x, 
-              test = "All")
-  )
-  
-  Wald_test_wt2 <- lapply(CR_types, function(x) 
-    Wald_test(mod_wt2, 
-              constraints = constrain_equal("Crit.Cat", reg_ex = TRUE),
-              vcov = x, 
-              test = "All")
-  )
-  
   expect_equal(vcovs_1, vcovs_2, tolerance = 1e-5)
-  compare_ttests(coef_test_wt1, coef_test_wt2, tol = 1e-5)
-  compare_Waldtests(Wald_test_wt1, Wald_test_wt2, tol = 1e-5)
-  
-  for (i in seq_along(vcovs_1)) {
-    expect_s3_class(vcovs_1[[i]], "vcovCR")
-    expect_s3_class(vcovs_2[[i]], "vcovCR")
-    expect_s3_class(coef_test_wt1[[i]], "coef_test_clubSandwich")
-    expect_s3_class(coef_test_wt2[[i]], "coef_test_clubSandwich")
-    expect_s3_class(Wald_test_wt1[[i]], "Wald_test_clubSandwich")
-    expect_s3_class(Wald_test_wt2[[i]], "Wald_test_clubSandwich")
-  }
-  
-  
+
 })
